--liquibase formatted sql

-- ============================================================================
--changeset hs-office-debitor-rbac-OBJECT:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRelatedRbacObject('hs_office_debitor');
--//


-- ============================================================================
--changeset hs-office-debitor-rbac-ROLE-DESCRIPTORS:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRbacRoleDescriptors('hsOfficeDebitor', 'hs_office_debitor');
--//


-- ============================================================================
--changeset hs-office-debitor-rbac-ROLES-CREATION:1 endDelimiter:--//
-- ----------------------------------------------------------------------------

/*
    Creates and updates the roles and their assignments for debitor entities.
 */

create or replace function hsOfficeDebitorRbacRolesTrigger()
    returns trigger
    language plpgsql
    strict as $$
declare
    debitorUuid         uuid;

    oldDebitorRel       hs_office_relationship;
    newDebitorRel       hs_office_relationship;

    newPartnerRel       hs_office_relationship;

    newBankAccount      hs_office_bankaccount;
    oldBankAccount      hs_office_bankaccount;

begin
    call enterTriggerForObjectUuid(NEW.uuid);

    debitorUuid := NEW.uuid;

    select * into newDebitorRel
            from hs_office_relationship as r where r.relType = 'ACCOUNTING' and r.relHolderUuid = NEW.debitorRelUuid;

    select * into newPartnerRel
            from hs_office_relationship as partnerRel
            where newDebitorRel.relAnchorUuid = partnerRel.relHolderUuid;

    select * into newBankAccount
             from hs_office_bankaccount as b where b.uuid = NEW.refundBankAccountUuid;

    if TG_OP = 'INSERT' then

        -- Permissions and Grants for Debitor

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipOwner(newDebitorRel)),
                createPermissions(partnerUuid, array ['DELETE'])
             );

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipAdmin(newDebitorRel), 'fail'),
                createPermissions(partnerUuid, array ['UPDATE'])
             );

        call grantPermissionsToRole(
                getRoleId(hsOfficeRelationshipTenant(newDebitorRel), 'fail'),
                createPermissions(partnerUuid, array ['SELECT'])
             );

        -- Grants to and from related Partner Relationship

--         call grantRoleToRole(hsOfficeRelationshipAdmin(newDebitorRel), hsOfficeRelationshipAdmin(newPartnerRel), true);
--         call grantRoleToRole(hsOfficeRelationshipAgent(newPartnerRel), hsOfficeRelationshipAdmin(newDebitorRel), true);
--
--         call grantRoleToRole(hsOfficeRelationshipAgent(newDebitorRel), hsOfficeRelationshipAgent(newPartnerRel), true);
--         call grantRoleToRole(hsOfficeRelationshipTenant(newPartnerRel), hsOfficeRelationshipAgent(newDebitorRel), true);

        -- Grants to and from refundBankAccount

--         if newBankAccount is not null then
--             call grantRoleToRole(hsOfficeBankAccountReferrer(newBankAccount), hsOfficeRelationshipAgent(newDebitorRel), true);
--             call grantRoleToRole(hsOfficeRelationshipAgent(newDebitorRel), hsOfficeBankAccountAdmin(newBankAccount), true);
--         end if;

    elsif TG_OP = 'UPDATE' then

        if OLD.debitorRelUuid is distinct from NEW.debitorRelUuid then

            select * into oldDebitorRel
                     from hs_office_relationship as r where r.relType = 'ACCOUNTING' and r.relHolderUuid = NEW.debitorRelUuid;

--             call grantPermissionsToRole(
--                     getRoleId(hsOfficeRelationshipOwner(newDebitorRel)),
--                     createPermissions(partnerUuid, array ['DELETE'])
--                  );
--
--             call grantPermissionsToRole(
--                     getRoleId(hsOfficeRelationshipAdmin(newDebitorRel)),
--                     createPermissions(partnerUuid, array ['UPDATE'])
--                  );
--
--             call grantPermissionsToRole(
--                     getRoleId(hsOfficeRelationshipTenant(newDebitorRel)),
--                     createPermissions(partnerUuid, array ['SELECT'])
--                  );

        end if;

        if OLD.refundBankAccountUuid is distinct from NEW.refundBankAccountUuid then

            select * into oldBankAccount
                from hs_office_bankaccount as b where b.uuid = OLD.refundBankAccountUuid;

            if oldBankAccount is not null then
                call revokeRoleFromRole(hsOfficeBankAccountReferrer(oldBankAccount), hsOfficeRelationshipAgent(oldDebitorRel), true);
                call revokeRoleFromRole(hsOfficeRelationshipAgent(oldDebitorRel), hsOfficeBankAccountAdmin(oldBankAccount), true);
            end if;

            if newBankAccount is not null then
                call grantRoleToRole(hsOfficeBankAccountReferrer(newBankAccount), hsOfficeRelationshipAgent(newDebitorRel), true);
                call grantRoleToRole(hsOfficeRelationshipAgent(newDebitorRel), hsOfficeBankAccountAdmin(newBankAccount), true);
            end if;
        end if;
    else
        raise exception 'invalid usage of TRIGGER';
    end if;

    call leaveTriggerForObjectUuid(NEW.uuid);
    return NEW;
end; $$;

/*
    An AFTER INSERT TRIGGER which creates the role structure for a new debitor.
 */
create trigger createRbacRolesForHsOfficeDebitor_Trigger
    after insert
    on hs_office_debitor
    for each row
execute procedure hsOfficeDebitorRbacRolesTrigger();

/*
    An AFTER UPDATE TRIGGER which updates the role structure of a debitor.
 */
create trigger updateRbacRolesForHsOfficeDebitor_Trigger
    after update
    on hs_office_debitor
    for each row
execute procedure hsOfficeDebitorRbacRolesTrigger();
--//


/*
    Creates and updates the roles and their assignments for debitor entities if partner rel changes.
 */

create or replace function hsOfficeDebitorPartnerRelRbacRolesTrigger()
    returns trigger
    language plpgsql
    strict as $$
declare

begin
    call enterTriggerForObjectUuid(NEW.uuid);

    -- TODO

    call leaveTriggerForObjectUuid(NEW.uuid);
    return NEW;
end; $$;
--//

/*
    An AFTER UPDATE TRIGGER which creates the role structure for debitors if partner relations change.
 */
create trigger updateRbacRolesForHsOfficeDebitor_Trigger
    after update
    on hs_office_partner
    for each row
execute procedure hsOfficeDebitorPartnerRelRbacRolesTrigger();
--//


-- ============================================================================
--changeset hs-office-debitor-rbac-IDENTITY-VIEW:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRbacIdentityViewFromProjection('hs_office_debitor', $idName$
    '#' || (select partner.partnerNumber
            from hs_office_partner partner
            join hs_office_relationship partnerRel on partnerRel.uuid = partner.partnerRoleUUid and partnerRel.relType = 'PARTNER'
            join hs_office_relationship debitorRel on debitorRel.relAnchorUuid = partnerRel.relHolderUuid and partnerRel.relType = 'ACCOUNTING'
            where debitorRel.uuid = target.debitorRelUuid)
         || to_char(debitorNumberSuffix, 'fm00')
    || ':' || (select split_part(idName, ':', 2) from hs_office_relationship_iv ri where ri.uuid = target.debitorRelUuid)
    $idName$);
--//


-- ============================================================================
--changeset hs-office-debitor-rbac-RESTRICTED-VIEW:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
call generateRbacRestrictedView('hs_office_debitor', 'target.debitorNumberSuffix',
    $updates$
        debitorRel = new.debitorRel,
        billable = new.billable,
        billingContactUuid = new.billingContactUuid,
        refundBankAccountUuid = new.refundBankAccountUuid,
        vatId = new.vatId,
        vatCountryCode = new.vatCountryCode,
        vatBusiness = new.vatBusiness,
        vatreversecharge = new.vatreversecharge,
        defaultPrefix = new.defaultPrefix -- TODO: Should it be allowed to updated this value?
    $updates$);
--//

-- ============================================================================
--changeset hs-office-debitor-rbac-NEW-DEBITOR:1 endDelimiter:--//
-- ----------------------------------------------------------------------------
/*
    Creates a global permission for new-debitor and assigns it to the hostsharing admins role.
 */
do language plpgsql $$
    declare
        addDebitorPermissions uuid[];
        globalObjectUuid      uuid;
        globalAdminRoleUuid   uuid ;
    begin
        call defineContext('granting global new-debitor permission to global admin role', null, null, null);

        globalAdminRoleUuid := findRoleId(globalAdmin());
        globalObjectUuid := (select uuid from global);
        addDebitorPermissions := createPermissions(globalObjectUuid, array ['new-debitor']);
        call grantPermissionsToRole(globalAdminRoleUuid, addDebitorPermissions);
    end;
$$;

/**
    Used by the trigger to prevent the add-debitor to current user respectively assumed roles.
 */
create or replace function addHsOfficeDebitorNotAllowedForCurrentSubjects()
    returns trigger
    language PLPGSQL
as $$
begin
    raise exception '[403] new-debitor not permitted for %',
        array_to_string(currentSubjects(), ';', 'null');
end; $$;

/**
    Checks if the user or assumed roles are allowed to create a new debitor.
 */
create trigger hs_office_debitor_insert_trigger
    before insert
    on hs_office_debitor
    for each row
    -- TODO.spec: who is allowed to create new debitors
    when ( not hasAssumedRole() )
execute procedure addHsOfficeDebitorNotAllowedForCurrentSubjects();
--//

