--liquibase formatted sql


-- ============================================================================
--changeset hs-office-membership-TEST-DATA-GENERATOR:1 endDelimiter:--//
-- ----------------------------------------------------------------------------

/*
    Creates a single membership test record.
 */
create or replace procedure createHsOfficeMembershipTestData(
        forPartnerNumber numeric(5),
        forMainDebitorNumberSuffix numeric,
        newMemberNumberSuffix char(2) )
    language plpgsql as $$
declare
    currentTask             varchar;
    relatedPartner          hs_office_partner;
    relatedDebitorRel       hs_office_relationship;
    relatedDebitor          hs_office_debitor;
begin
    currentTask := 'creating Membership test-data ' ||
                    'P-' || forPartnerNumber::text ||
                    'D-...' || forMainDebitorNumberSuffix ||
                    'M-...' || newMemberNumberSuffix;
    call defineContext(currentTask, null, 'superuser-alex@hostsharing.net', 'global#global.admin');
    execute format('set local hsadminng.currentTask to %L', currentTask);

    select partner.* from hs_office_partner partner
                     where partner.partnerNumber = forPartnerNumber into relatedPartner;
    select debitorRel.* from hs_office_relationship debitorRel
                join hs_office_relationship partnerRel
                    on debitorRel.relAnchorUuid=partnerRel.relHolderUuid and partnerRel.relType='PARTNER'
                join hs_office_partner partner
                    on partner.partnerRoleUuid = partnerRel.uuid
                where debitorRel.relType='ACCOUNTING' -- FIXME: 'DEBITOR'
                into relatedDebitorRel;
    select debitor.* from hs_office_debitor debitor
               where debitor.debitorRelUuid = relatedDebitorRel.uuid
                   and debitor.debitorNumberSuffix = forMainDebitorNumberSuffix
               into relatedDebitor;

    raise notice 'creating test Membership: M-% %', forPartnerNumber, newMemberNumberSuffix;
    raise notice '- using partner (%): %', relatedPartner.uuid, relatedPartner;
    raise notice '- using debitor (%): %', relatedDebitor.uuid, relatedDebitor;
    insert
        into hs_office_membership (uuid, partneruuid, maindebitoruuid, memberNumberSuffix, validity, reasonfortermination)
        values (uuid_generate_v4(), relatedPartner.uuid, relatedDebitor.uuid, newMemberNumberSuffix, daterange('20221001' , null, '[]'), 'NONE');
end; $$;
--//


-- ============================================================================
--changeset hs-office-membership-TEST-DATA-GENERATION:1 –context=dev,tc endDelimiter:--//
-- ----------------------------------------------------------------------------

do language plpgsql $$
    begin
        call createHsOfficeMembershipTestData(10001, 11, '01');
        call createHsOfficeMembershipTestData(10002, 12, '02');
        call createHsOfficeMembershipTestData(10003, 13, '03');
    end;
$$;
--//
